@@ -5,26 +5,54 @@
 #' @param var_correct A logical indicating whether to return the corrected prediction
 #'   variances when predicting via models fit using \code{ssn_glm}. The default is
 #'   \code{TRUE}.
+#' @param dispersion The dispersion of assumed when computing the prediction standard errors
+#'   for \code{ssn_glm()} model objects when \code{family}
+#'   is \code{"nbinomial"}, \code{"beta"}, \code{"Gamma"}, or \code{"inverse.gaussian"}.
+#'   If omitted, the model object dispersion parameter is used.
 #' @rdname predict.SSN2
 #' @method predict ssn_glm
 #' @export
-predict.ssn_glm <- function(object, newdata, type = c("link", "response"), se.fit = FALSE, interval = c("none", "confidence", "prediction"),
-                            newdata_size, level = 0.95, var_correct = TRUE, ...) {
+predict.ssn_glm <- function(object, newdata, type = c("link", "response", "terms"), se.fit = FALSE, interval = c("none", "confidence", "prediction"),
+                            level = 0.95, dispersion = NULL, terms = NULL, local, var_correct = TRUE, newdata_size, na.action = na.fail, ...) {
   # match type argument so the two display
   type <- match.arg(type)
 
   # match interval argument so the three display
   interval <- match.arg(interval)
 
+  # new data name
+  if (missing(newdata)) newdata <- "all"
+  newdata_name <- newdata
+
   # deal with newdata_size
   if (missing(newdata_size)) newdata_size <- NULL
 
+  # handle dispersion argument if provided
+  if (!is.null(dispersion)) {
+    if (object$family %in% c("binomial", "poisson") && dispersion != 1) {
+      stop("dispersion is fixed at one for binomial and poisson families.", call. = FALSE)
+    }
+    object$coefficients$params_object$dispersion[1] <- dispersion
+  }
+
   # handle local for now
-  local <- FALSE
+  if (missing(local)) local <- NULL
+  # deal with local
+  if (is.null(local)) {
+    if (newdata != "all") {
+      if (object$n > 10000 || (object$n * NROW(object$ssn.object$preds[[newdata]]) > 1e8)) {
+        # if (object$n > 5000 || NROW(newdata) > 5000) {
+        local <- TRUE
+        message("Because the large sample size or number of predictions, we are setting local = TRUE to perform computationally efficient approximations. To override this behavior and compute the exact solution, rerun predict() with local = FALSE. Be aware that setting local = FALSE may result in exceedingly long computational times.")
+      } else {
+        local <- FALSE
+      }
+    }
+  }
+  # make local list
+  local_list <- get_local_list_prediction(local)
+  local <- local_list
 
-  # new data name
-  if (missing(newdata)) newdata <- NULL
-  newdata_name <- newdata
   # iterate through prediction names
   if (is.null(newdata_name) || newdata_name == "all") {
     newdata_name <- names(object$ssn.object$preds)
@@ -58,6 +86,13 @@ predict.ssn_glm <- function(object, newdata, type = c("link", "response"), se.fi
 
   # make covariance object
   cov_vector <- covmatrix(object, newdata_name)
+  if (local_list$method == "covariance") {
+    cov_vector_means <- colMeans(cov_vector)
+    cov_index <- order(as.numeric(cov_vector_means))[seq(from = object$n, to = max(1, object$n - local$size + 1))]
+    cov_vector <- cov_vector[, cov_index, drop = FALSE]
+  } else {
+    cov_index <- NULL
+  }
   cov_vector_list <- split(cov_vector, seq_len(NROW(cov_vector)))
 
   formula_newdata <- delete.response(terms(object))
@@ -89,6 +124,13 @@ predict.ssn_glm <- function(object, newdata, type = c("link", "response"), se.fi
   attr(newdata_model, "assign") <- attr_assign[keep_cols]
   attr(newdata_model, "contrasts") <- attr_contrasts
 
+  # call terms if needed
+  if (type == "terms") {
+    # glm supports standard errors for terms objects but not intervals (no interval argument)
+    # scale df not used for glms
+    return(predict_terms(object, newdata_model, se.fit, scale = NULL, df = Inf, interval, level, add_newdata_rows, terms, ...))
+  }
+
   # storing newdata as a list
   newdata_rows_list <- split(newdata, seq_len(NROW(newdata)))
 
@@ -110,60 +152,92 @@ predict.ssn_glm <- function(object, newdata, type = c("link", "response"), se.fi
   if (interval %in% c("none", "prediction")) {
     local_list <- get_local_list_prediction(local)
 
-    # matrix cholesky
     if (local_list$method == "all") {
-      cov_matrix_val <- covmatrix(object)
       cov_lowchol <- t(chol(cov_matrix_val))
       predvar_adjust_ind <- FALSE
       predvar_adjust_all <- TRUE
     } else {
-      cov_lowchol <- NULL
-      predvar_adjust_ind <- TRUE
-      predvar_adjust_all <- FALSE
+      cov_matrix_val <- cov_matrix_val[cov_index, cov_index, drop = FALSE]
+      cov_lowchol <- t(chol(cov_matrix_val))
+      # predvar_adjust_ind <- TRUE
+      # predvar_adjust_all <- FALSE
+      predvar_adjust_ind <- FALSE
+      predvar_adjust_all <- TRUE # changed to this with averaging subsetting
     }
 
+    # # matrix cholesky
+    # if (local_list$method == "all") {
+    #   cov_matrix_val <- covmatrix(object)
+    #   cov_lowchol <- t(chol(cov_matrix_val))
+    #   predvar_adjust_ind <- FALSE
+    #   predvar_adjust_all <- TRUE
+    # } else {
+    #   cov_lowchol <- NULL
+    #   predvar_adjust_ind <- TRUE
+    #   predvar_adjust_all <- FALSE
+    # }
+
     # change predvar adjust based on var correct
     if (!var_correct) {
       predvar_adjust_ind <- FALSE
       predvar_adjust_all <- FALSE
     }
 
+    Xmat <- model.matrix(object)
+    y <- model.response(model.frame(object))
+    offset <- model.offset(model.frame(object))
+    w <- fitted(object, type = "link")
+    size <- object$size
+    if (!is.null(cov_index)) {
+      Xmat <- Xmat[cov_index, , drop = FALSE]
+      y <- y[cov_index]
+      w <- w[cov_index]
+      if (!is.null(offset)) {
+        offset <- offset[cov_index]
+      }
+      if (!is.null(size)) {
+        size <- size[cov_index]
+      }
+    }
+
     # until big data back
-    # if (local_list$parallel) {
-    #   cl <- parallel::makeCluster(local_list$ncores)
-    #   pred_val <- parallel::parLapply(cl, newdata_list, get_pred_glm,
-    #                                   se.fit = se.fit, interval = interval, formula = object$formula,
-    #                    obdata = obdata, cov_matrix_val = cov_matrix_val, total_var = total_var, cov_lowchol = cov_lowchol,
-    #                    Xmat = model.matrix(object), y = model.response(model.frame(object)),
-    #                    betahat = coefficients(object), cov_betahat = vcov(object, var_correct = FALSE),
-    #                    contrasts = object$contrasts, local = local_list,
-    #                    family = object$family, w = fitted(object, type = "link"),
-    #                    size = object$size, dispersion = dispersion_params_val,
-    #                    predvar_adjust_ind = predvar_adjust_ind, xlevels = object$xlevels)
-    #   cl <- parallel::stopCluster(cl)
-    # } else {
-    #   pred_val <- lapply(newdata_list, get_pred_glm,
-    #                    se.fit = se.fit, interval = interval, formula = object$formula,
-    #                    obdata = obdata, cov_matrix_val = cov_matrix_val, total_var = total_var, cov_lowchol = cov_lowchol,
-    #                    Xmat = model.matrix(object), y = model.response(model.frame(object)),
-    #                    betahat = coefficients(object), cov_betahat = vcov(object, var_correct = FALSE),
-    #                    contrasts = object$contrasts, local = local_list,
-    #                    family = object$family, w = fitted(object, type = "link"),
-    #                    size = object$size, dispersion = dispersion_params_val,
-    #                    predvar_adjust_ind = predvar_adjust_ind, xlevels = object$xlevels)
+    if (local_list$parallel) {
+      cl <- parallel::makeCluster(local_list$ncores)
+      pred_val <- parallel::parLapply(cl, newdata_list, get_pred_glm,
+                                      se.fit = se.fit, interval = interval, formula = object$formula,
+                       obdata = obdata, cov_matrix_val = cov_matrix_val, total_var = total_var, cov_lowchol = cov_lowchol,
+                       Xmat = Xmat, y = y,
+                       betahat = coefficients(object), cov_betahat = vcov(object, var_correct = FALSE),
+                       contrasts = object$contrasts, local = local_list,
+                       family = object$family, w = w,
+                       size = size, dispersion = dispersion_params_val,
+                       predvar_adjust_ind = predvar_adjust_ind, xlevels = object$xlevels, cov_index = cov_index)
+      cl <- parallel::stopCluster(cl)
+    } else {
+      pred_val <- lapply(newdata_list, get_pred_glm,
+                       se.fit = se.fit, interval = interval, formula = object$formula,
+                       obdata = obdata, cov_matrix_val = cov_matrix_val, total_var = total_var, cov_lowchol = cov_lowchol,
+                       Xmat = Xmat, y = y,
+                       betahat = coefficients(object), cov_betahat = vcov(object, var_correct = FALSE),
+                       contrasts = object$contrasts, local = local_list,
+                       family = object$family, w = w,
+                       size = size, dispersion = dispersion_params_val,
+                       predvar_adjust_ind = predvar_adjust_ind, xlevels = object$xlevels, cov_index = cov_index)
+
+    }
+
+    # pred_val <- lapply(newdata_list, get_pred_glm,
+    #   se.fit = se.fit, interval = interval, formula = object$formula,
+    #   obdata = obdata, cov_matrix_val = cov_matrix_val, total_var = total_var, cov_lowchol = cov_lowchol,
+    #   Xmat = model.matrix(object), y = model.response(model.frame(object)),
+    #   betahat = coefficients(object), cov_betahat = vcov(object, var_correct = FALSE),
+    #   contrasts = object$contrasts, local = local_list,
+    #   family = object$family, w = fitted(object, type = "link"),
+    #   size = object$size, dispersion = dispersion_params_val,
+    #   predvar_adjust_ind = predvar_adjust_ind, xlevels = object$xlevels, cov_index
+    # )
 
-    # }
 
-    pred_val <- lapply(newdata_list, get_pred_glm,
-      se.fit = se.fit, interval = interval, formula = object$formula,
-      obdata = obdata, cov_matrix_val = cov_matrix_val, total_var = total_var, cov_lowchol = cov_lowchol,
-      Xmat = model.matrix(object), y = model.response(model.frame(object)),
-      betahat = coefficients(object), cov_betahat = vcov(object, var_correct = FALSE),
-      contrasts = object$contrasts, local = local_list,
-      family = object$family, w = fitted(object, type = "link"),
-      size = object$size, dispersion = dispersion_params_val,
-      predvar_adjust_ind = predvar_adjust_ind, xlevels = object$xlevels
-    )
 
 
 
@@ -185,13 +259,15 @@ predict.ssn_glm <- function(object, newdata, type = c("link", "response"), se.fi
           vars_adj <- get_wts_varw(
             family = object$family,
             Xmat = model.matrix(object),
-            y = object$y,
+            y = model.response(model.frame(object)),
             w = fitted(object, type = "link"),
             size = object$size,
             dispersion = dispersion_params_val,
             cov_lowchol = cov_lowchol,
             x0 = newdata_model,
-            c0 = cov_vector
+            c0 = cov_vector,
+            cov_index = cov_index,
+            cov_betahat = vcov(object, var_correct = FALSE)
           )
           vars <- vars_adj + vars
         }
@@ -220,13 +296,15 @@ predict.ssn_glm <- function(object, newdata, type = c("link", "response"), se.fi
         vars_adj <- get_wts_varw(
           family = object$family,
           Xmat = model.matrix(object),
-          y = object$y,
+          y = model.response(model.frame(object)),
           w = fitted(object, type = "link"),
           size = object$size,
           dispersion = dispersion_params_val,
           cov_lowchol = cov_lowchol,
           x0 = newdata_model,
-          c0 = cov_vector
+          c0 = cov_vector,
+          cov_index = cov_index,
+          cov_betahat = vcov(object, var_correct = FALSE)
         )
         vars <- vars_adj + vars
       }
@@ -325,24 +403,42 @@ predict.ssn_glm <- function(object, newdata, type = c("link", "response"), se.fi
 get_pred_glm <- function(newdata_list, se.fit, interval,
                          formula, obdata, cov_matrix_val, total_var, cov_lowchol,
                          Xmat, y, betahat, cov_betahat, contrasts, local,
-                         family, w, size, dispersion, predvar_adjust_ind, xlevels) {
+                         family, w, size, dispersion, predvar_adjust_ind, xlevels, cov_index) {
   cov_vector_val <- newdata_list$c0
 
-  if (local$method == "covariance") {
-    # n <- length(cov_vector_val)
-    # cov_index <- order(as.numeric(cov_vector_val))[seq(from = n, to = max(1, n - local$size + 1))] # use abs() here?
-    # obdata <- obdata[cov_index, , drop = FALSE]
-    # cov_vector_val <- cov_vector_val[cov_index]
-    # cov_matrix_val <- cov_matrix_val[cov_index, cov_index, drop = FALSE]
-    # w <- w[cov_index]
-    # y <- y[cov_index]
-    # if (!is.null(size)) {
-    #   size <- size[cov_index]
-    # }
-    # cov_lowchol <- t(Matrix::chol(Matrix::forceSymmetric(cov_matrix_val)))
-    # model_frame <- model.frame(formula, obdata, drop.unused.levels = TRUE, na.action = na.pass, xlev = xlevels)
-    # Xmat <- model.matrix(formula, model_frame, contrasts = contrasts)
-  }
+  # moved indexing of relevant quantities outside the function (cov_index no longer needed)
+  # if (!is.null(cov_index)) {
+  #   obdata <- obdata[cov_index, , drop = FALSE]
+  #   model_frame <- model.frame(formula, obdata, drop.unused.levels = TRUE, na.action = na.pass, xlev = xlevels)
+  #   Xmat <- model.matrix(formula, model_frame, contrasts = contrasts)
+  #   # Xmat <- Xmat[cov_index, , drop = FALSE]
+  #   y <- model.response(model_frame)
+  #   # y <- y[cov_index]
+  #   offset <- model.offset(model_frame)
+  #   # if (!is.null(offset)) {
+  #   #   offset <- offset[cov_index]
+  #   # }
+  #   w <- w[cov_index]
+  #   if (!is.null(size)) {
+  #     size <- size[cov_index]
+  #   }
+  # }
+
+  # if (local$method == "covariance") {
+  #   # n <- length(cov_vector_val)
+  #   # cov_index <- order(as.numeric(cov_vector_val))[seq(from = n, to = max(1, n - local$size + 1))] # use abs() here?
+  #   # obdata <- obdata[cov_index, , drop = FALSE]
+  #   # cov_vector_val <- cov_vector_val[cov_index]
+  #   # cov_matrix_val <- cov_matrix_val[cov_index, cov_index, drop = FALSE]
+  #   # w <- w[cov_index]
+  #   # y <- y[cov_index]
+  #   # if (!is.null(size)) {
+  #   #   size <- size[cov_index]
+  #   # }
+  #   # cov_lowchol <- t(Matrix::chol(Matrix::forceSymmetric(cov_matrix_val)))
+  #   # model_frame <- model.frame(formula, obdata, drop.unused.levels = TRUE, na.action = na.pass, xlev = xlevels)
+  #   # Xmat <- model.matrix(formula, model_frame, contrasts = contrasts)
+  # }
 
   c0 <- as.numeric(cov_vector_val)
   SqrtSigInv_X <- forwardsolve(cov_lowchol, Xmat)
@@ -356,7 +452,7 @@ get_pred_glm <- function(newdata_list, se.fit, interval,
   if (se.fit || interval == "prediction") {
     var <- as.numeric(total_var - Matrix::crossprod(SqrtSigInv_c0, SqrtSigInv_c0) + H %*% Matrix::tcrossprod(cov_betahat, H))
     if (predvar_adjust_ind) {
-      var_adj <- get_wts_varw(family, Xmat, y, w, size, dispersion, cov_lowchol, x0, c0)
+      var_adj <- get_wts_varw(family, Xmat, y, w, size, dispersion, cov_lowchol, x0, c0, cov_index = cov_index, cov_betahat = cov_betahat)
       var <- var_adj + var
     }
     pred_list <- list(fit = fit, var = var)
@@ -379,10 +475,22 @@ get_pred_glm <- function(newdata_list, se.fit, interval,
 #' @param c0 Covariance between observed and newdata
 #'
 #' @noRd
-get_wts_varw <- function(family, Xmat, y, w, size, dispersion, cov_lowchol, x0, c0) {
+get_wts_varw <- function(family, Xmat, y, w, size, dispersion, cov_lowchol, x0, c0, cov_index, cov_betahat) {
+
+
   SigInv <- chol2inv(t(cov_lowchol)) # works on upchol
+
+  if (!is.null(cov_index)) {
+    Xmat <- Xmat[cov_index, , drop = FALSE]
+    y <- y[cov_index]
+    w <- w[cov_index]
+    if (!is.null(size)) {
+      size <- size[cov_index]
+    }
+  }
   SigInv_X <- SigInv %*% Xmat
-  cov_betahat <- chol2inv(chol(Matrix::forceSymmetric(crossprod(Xmat, SigInv_X))))
+  # cov_betahat <- chol2inv(chol(Matrix::forceSymmetric(crossprod(Xmat, SigInv_X)))) # invertibility issues big data
+  cov_betahat <- cov_betahat
   wts_beta <- tcrossprod(cov_betahat, SigInv_X)
   Ptheta <- SigInv - SigInv_X %*% wts_beta
 
